"""Definition of functions that are used to interact with youtube."""

from urllib.request import urlopen
import requests
from bs4 import BeautifulSoup
import os
import youtube_dl
from ytmdl import defaults


def GRAB_SONG(link):
    """Return true if the song is downloaded else false."""
    ydl_opts = {
        'format': 'bestaudio',
        'quiet': True,
        'outtmpl': os.path.join(defaults.DEFAULT.SONG_TEMP_DIR,
                                '%(title)s.%(ext)s'),
        'postprocessors': [{
            'key': 'FFmpegExtractAudio',
            'preferredcodec':  'mp3',
            'preferredquality': defaults.DEFAULT.SONG_QUALITY
        }]
    }

    # Download the song with youtube-dl
    try:
        ydl = youtube_dl.YoutubeDL(ydl_opts)
        ydl.download([link])
        return True
    except TimeoutError:
        print('Timed Out! Are you connected to internet?\a')
        return False
    else:
        return False


video = []
urls = []


def get_href(url):
    """Get the watch? part of the url in case of urls."""
    pos_watch = url.index('/watch?v=')

    part = url[pos_watch:]

    return part


def search(querry, lim=5):
    """Search the querry in youtube and return lim number of results.

    Querry is the keyword, i:e name of the song
    lim is the number of songs that will be added to video array and returned
    """
    # Replace all the spaces with +
    querry = querry.replace(' ', '+')

    url = "https://www.youtube.com/results?search_query={}".format(querry)

    response = urlopen(url)
    html = response.read()
    soup = BeautifulSoup(html, "html.parser")
    count = 0
    for vid in soup.findAll(attrs={'class': 'yt-uix-tile-link'}):
        if lim == count:
            break

        url = vid['href']

        try:
            data = scan_video(url)
        except Exception:
            # If something was wrong, append the last result and return
            video.append(data)
            urls.append(url)
            break

        video.append(data)
        urls.append(url)
        count += 1
    return (video, urls)


def scan_video(url):
    """Scan the link of the video and return data and."""
    search_tmplt = "http://www.youtube.com/oembed?url={}&format=json"
    search_url = search_tmplt.format(url)
    data = requests.get(search_url).json()

    return data



"""Functions used in setting up the config file are defined here."""

import os
import shutil

config_text = '#*****************************************#\
                #*-------------config for ytmdl ----------#\
                #\
                #-----------------------------------------#\
                #------PLEASE DONT LEAVE ANY BLANK LINE---#\
                #-----------------------------------------#\
                #\
                # To change defaults just remove the hash(#)\
                # from thw beginning of the line.\
                # The will be read as a single line comment.\
                #\
                #*****************************************\
                # The SONG_DIR is the directory where all the songs will be saved.\
                # In order to change it, simply remove the hash from beginning\
                # And change the path to your desired one.\
                # In case the path has spaces in in, include it in a " "\
                # Following is a simple folder path example\
                #\
                #SONG_DIR = "path/to/your/desired/folder"\
                #\
                #************--------ADVANCED-------*********\
                # If you want to save the song in custom folders than those can be\
                # added to the name like the following example.\
                # The possible values are following\
                #\
                # Artist --> Song Artist\
                # Album  --> Song Album Name\
                # Title  --> Song Name\
                # Genre  --> Song Genre\
                # TrackNumber --> Song Number in the album\
                # ReleaseDate --> Song Release date\
                #\
                # Following is an example of the format\
                #SONG_DIR = "/home/deepjyoti30/Music$Artist->Album->Title"\
                #\
                #*****************************************#\
                # The QUALITY is the quality of the song in kbps\
                # By default it is set to 320kbps\
                # In case you want to change it to something else,\
                # Uncomment the following line and change it\
                #\
                # Supported values are 320 and 192\
                #\
                #QUALITY = "320"\
                #'


class DEFAULTS:
    """Some default stuff defined."""

    # The home dir
    HOME_DIR = os.path.expanduser('~')

    # The default song dir
    SONG_DIR = os.path.join(HOME_DIR, 'Music')

    # The temp dir
    SONG_TEMP_DIR = os.path.join(SONG_DIR, 'ytmdl')

    # The default song quality
    SONG_QUALITY = '320'

    # The config path
    CONFIG_PATH = os.path.join(HOME_DIR, '.config', 'ytmdl')


def make_config():
    """Copy the config file to .config folder."""
    # Remove the current config from SONG_TEMP_DIR
    config_path = os.path.join(DEFAULTS.CONFIG_PATH, 'config')

    # Check if the ytmdl folder is present in config
    if not os.path.isdir(DEFAULTS.CONFIG_PATH):
        # Make the ytmdl folder
        os.mkdir(DEFAULTS.CONFIG_PATH)
    elif os.path.isfile(config_path):
        os.remove(config_path)

    # Now write the config test to config file
    with open(config_path, 'w') as write_config:
        write_config.write(config_text)


def checkConfig():
    """Need to check the config to see if defaults are changed.

    The config will be saved in the .config folder.
    """
    # Try to see if the config is present in the SONG_TEMP_DIR

    if os.path.isdir(DEFAULTS.CONFIG_PATH):
        DIR_CONTENTS = os.listdir(DEFAULTS.CONFIG_PATH)
    else:
        return False

    if 'config' not in DIR_CONTENTS:
        make_config()
        return True
    else:
        return True


def checkExistence(keyword, value):
    """Check if the user specified value in config is possible."""
    if keyword == 'SONG_DIR':
        # In this case check if $ and -> are presnt
        # If they are then only check if the base dir exists
        if '$' in value and '->' in value:
            pos = value.find('$')
            value = value[:pos]

        if os.path.isdir(value):
            return True
        else:
            return False
    elif keyword == 'QUALITY':
        # Possible values that QUALITY can take
        possQ = ['320', '192']

        if value in possQ:
            return True
        else:
            return False


def retDefault(keyword):
    """Return the DEFAULT value of keyword."""
    if keyword == 'QUALITY':
        return DEFAULTS.SONG_QUALITY
    elif keyword == 'SONG_DIR':
        return DEFAULTS.SONG_DIR


def GIVE_DEFAULT(self, keyword):
    """Check if the user has uncommented the config and added something.

    If possible get what is changed, else return the default value.
    """
    # Check If the config is already present in SONG_TEMP_DIR
    if not checkConfig():
        return retDefault(keyword)
    else:
        # Then read from it
        READ_STREAM = open(os.path.join(DEFAULTS.CONFIG_PATH, 'config'), 'r')

        while True:
            line = READ_STREAM.readline()
            if not line:
                return retDefault(keyword)
            if line[0] != '#' and keyword in line:
                # Get the position of =
                index_equal = line.index('=')
                if line[index_equal + 1] == ' ':
                    newDEFAULT = line[index_equal + 2:]
                else:
                    newDEFAULT = line[index_equal + 1:]

                # Remove the "
                newDEFAULT = newDEFAULT.replace('"', '')
                # Check if the line has a \n in it
                if "\n" in line:
                    newDEFAULT = newDEFAULT.replace('\n', '')

                if checkExistence(keyword, newDEFAULT):
                    return newDEFAULT
                else:
                    return retDefault(keyword)


if __name__ == '__main__':
    make_config()
    exit(0)
